--[[ 
Bill's Utils

This addon contains functions used in several of my addons so in order to avoid duplicate code
I moved them to this utility library 

This library will wipe out it's globals after 5 mins so make sure to reference them in local variables
in the addons that need them any unreferenced functions will be garbage collected afterwards to minimize
the memory footprint ]]

local on = true
local off = false

BillsUtils = {}

-- Wait/Delay function
local waitTable = {};
local waitFrame = nil;

BillsUtils.Wait = function(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

-- Add Meta Tables to Variables
BillsUtils.addOptionMt = function(options, defaults)
	setmetatable(options, {__index = defaults})
	for i, v in pairs(options) do
		if type(v) == "table" and not getmetatable(v) then
			addOptionMt(v, defaults[i])
		end
	end
end

-- Return Colored status
BillsUtils.StatColor = function(stat)
	local vtype = type(stat)
	if vtype == "boolean" then
		if stat == on then
			return "|cFF00FF00[on]"
		elseif stat == off then
			return "|cFFFF0000[off]"
		end
	elseif vtype == "number" then
		if stat == 0 then
			return "|cFFFF0000[0] (DISABLED)"
		elseif stat > 0 and stat <= 60 then
			return string.format("|cFF00FF00[%i] second(s)", stat)
		end
	end
end

-- String Join print
BillsUtils.SJprint = function(...)
	print( string.join(" ", ...))
end

-- Sorts 2 tables where x in both tables are related
BillsUtils.LinkedSort = function( main, linked )
	if #main == 0 or # linked == 0 then
		return false
	end
	local sorted = false
	
	while sorted == false do
		local changed = false
		for x = #main -1, 1, -1 do
			if main[x] > main[x+1] then
				main[x], main[x+1] = main[x+1], main[x]
				linked[x], linked[x+1] = linked[x+1], linked[x]
				changed = true
			end
		end
		if not(changed) then
			sorted = true
		end
	end	
end

-- Checks CmdLine switches agains variable and changes var if needed true/false
BillsUtils.OptSaveTF = function( Table, Key, Value)
	local Saved = Table[ Key ]
	if Value == "on" or Value == "off" then
		if Saved ~= Value then
			Table[ Key ] = Value == "on"
		end
	end
end

-- Checks CmdLine switches agains variable and changes var if needed Number
BillsUtils.OptSaveVal = function( Table, Key, Value, Min, Max)
	local Saved = Table[ Key ]
	if Value == nil then
		return
	end
	if Value ~= Saved then
		if Value >= Min and Value <= Max then
			Table[ Key ] = Value
		end
	end
end

-- Checks checkboxes against variable and adjusts variable if needed
BillsUtils.ChkBoxSave = function( Panel, Box, Table, Key)
	if Key == nil then
		Key = Box
	end
	
	local checked = Panel[ Box ]:GetChecked() == 1
	local saved = Table[Key]
	
	if checked ~= saved then
		Table[ Key ] = checked
	end
end

-- Checks Slider against variable and changes var if needed
BillsUtils.SliderSave = function( Panel, Slider, Table, Key)
	if Key == nil then
		Key = Slider
	end
	local SliderValue = Panel[Slider]:GetValue()
	local saved = Table[Key]
	
	if SliderValue ~= saved then
		Table[ Key ] = SliderValue
	end
end

-- Mousewheel control for sliders
BillsUtils.SliderMW = function(self, delta)
	local min,max = self:GetMinMaxValues()
	local step = self:GetValueStep()
	local value = self:GetValue()
	
	if (value > min) and (delta < 0) then 
		value = (value - step)
		if (value > min) then 
			self:SetValue( value )
		else
			self:SetValue( min )
		end
	end
	if (value < max) and (delta > 0) then
		value = (value + step)
		if (value < max) then
			self:SetValue( value )
		else
			self:SetValue( max )
		end
	end
end

-- Frame Event Registration Toggler
BillsUtils.EventToggle = function( Frame, Table, Key, Event, Enable )
	if not(Enable) then
		if Frame:IsEventRegistered(Event) then
			Frame:UnregisterEvent( Event )
		end
		return
	end
	
	if Frame:IsEventRegistered(Event) ~= Table[Key] then
		if Table[Key] then
			Frame:RegisterEvent( Event )
		else
			Frame:UnregisterEvent( Event )
		end
	end
end

-- Line below minimizes mem usage by dropping unused functions
-- 5 mins after loading
BillsUtils.Wait(300, table.wipe, BillsUtils)